@@ -1,11 +1,10 @@
+from __future__ import print_function
+
 import os
 import stat
-import fcntl
-import errno
 import pwd
 import grp
 import shutil
-import hashlib
 import tarfile
 import pkg_resources
 import argparse
@@ -37,27 +36,26 @@ class BaseRunner(object):
 
         args = parser.parse_args()
 
-        # We intentionally don't close the file.  We leave it open and grab a lock
-        # to ensure that two runners aren't trying to run the same proc. 
-        self.file = open(args.file, 'rwb')
-
-        self.config = ProcData(yaml.safe_load(self.file))
-
         try:
             cmd = self.commands[args.command]
-            # Commands that have a lock=False attribute won't try to lock the
-            # proc.yaml file.  'uptest' and 'shell' are in this category.
-            if getattr(cmd, 'lock', True):
-                lock_file(self.file)
-            else:
-                self.file.close()
         except KeyError:
             raise SystemExit("Command must be one of: %s" %
                              ', '.join(self.commands.keys()))
-        cmd()
+
+        with open(args.file, 'r+b') as file:
+            self.config = ProcData(yaml.safe_load(file))
+
+            # Lock the file for exclusive access. Some commands (such as shell
+            # or uptest) may override the behavior by providing a 'lock'
+            # attribute on the method.
+            getattr(cmd, 'lock', lock_file)(file)
+            cmd()
+
+    def __close_file(file):
+        file.close()
 
     def setup(self):
-        print "Setting up", get_container_name(self.config)
+        print("Setting up", get_container_name(self.config))
         self.make_proc_dirs()
         self.ensure_build()
         self.write_proc_lxc()
@@ -66,19 +64,19 @@ class BaseRunner(object):
         self.write_env_sh()
 
     def run(self):
-        print "Running", get_container_name(self.config)
+        print("Running", get_container_name(self.config))
         args = self.get_lxc_args()
         os.execve(which('lxc-start')[0], args, {})
 
     def shell(self):
-        print "Running shell for", get_container_name(self.config)
+        print("Running shell for", get_container_name(self.config))
         args = self.get_lxc_args(special_cmd='/bin/bash')
         os.execve(which('lxc-start')[0], args, {})
-    shell.lock = False
+    shell.lock = __close_file
 
     def untar(self):
         tarpath = get_buildfile_path(self.config)
-        print "Untarring", tarpath
+        print("Untarring", tarpath)
         outfolder = get_app_path(self.config)
         owners = (self.config.user, self.config.group)
         untar(tarpath, outfolder, owners)
@@ -88,7 +86,7 @@ class BaseRunner(object):
         Write the script that is the first thing called inside the container.  It
         sets env vars and then calls the real program.
         """
-        print "Writing proc.sh"
+        print("Writing proc.sh")
         context = {
             'tmp': '/tmp',
             'home': '/app',
@@ -99,16 +97,16 @@ class BaseRunner(object):
         }
         sh_path = os.path.join(get_container_path(self.config), 'proc.sh')
         rendered = get_template('proc.sh') % context
-        with open(sh_path, 'wb') as f:
+        with open(sh_path, 'w') as f:
             f.write(rendered)
         st = os.stat(sh_path)
         os.chmod(sh_path, st.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
 
     def write_env_sh(self):
-        print "Writing env.sh"
+        print("Writing env.sh")
         envsh_path = os.path.join(get_container_path(self.config), 'env.sh')
 
-        with open(envsh_path, 'wb') as f:
+        with open(envsh_path, 'w') as f:
             def format_var(key, val):
                 return 'export %s="%s"' % (key, val)
             e = self.config.env
@@ -126,7 +124,7 @@ class BaseRunner(object):
             return self.config.cmd
 
         procfile_path = os.path.join(get_app_path(self.config), 'Procfile')
-        with open(procfile_path, 'rb') as f:
+        with open(procfile_path, 'r') as f:
             procs = yaml.safe_load(f)
         return procs[self.config.proc_name]
 
@@ -144,13 +142,12 @@ class BaseRunner(object):
             ensure_file(self.config.build_url, path, build_md5)
 
             # Now untar.
-            outfolder = get_app_path(self.config)
             self.untar()
 
     def write_settings_yaml(self):
-        print "Writing settings.yaml"
+        print("Writing settings.yaml")
         path = os.path.join(get_container_path(self.config), 'settings.yaml')
-        with open(path, 'wb') as f:
+        with open(path, 'w') as f:
             f.write(yaml.safe_dump(self.config.settings, default_flow_style=False))
 
     def get_lxc_args(self, special_cmd=None):
@@ -208,8 +205,8 @@ class BaseRunner(object):
                 os.execve(which('lxc-start')[0], args, {})
             else:
                 # There are no uptests for this proc.  Output an empty JSON list.
-                print "[]"
-    uptest.lock = False
+                print("[]")
+    uptest.lock = __close_file
 
     def teardown(self):
         # Everything should have been put in the proc path, so delete that.
@@ -218,7 +215,7 @@ class BaseRunner(object):
         shutil.rmtree(get_proc_path(self.config))
 
     def make_proc_dirs(self):
-        print "Making directories"
+        print("Making directories")
         proc_path = get_proc_path(self.config)
         container_path = get_container_path(self.config)
         mkdir(proc_path)
@@ -229,7 +226,7 @@ class BaseRunner(object):
             mkdir(os.path.join(container_path, inside.lstrip('/')))
 
     def write_proc_lxc(self):
-        print "Writing proc.lxc"
+        print("Writing proc.lxc")
 
         proc_path = get_proc_path(self.config)
         container_path = get_container_path(self.config)
@@ -243,7 +240,7 @@ class BaseRunner(object):
         content += self.get_lxc_volume_str()
 
         filepath = os.path.join(proc_path, 'proc.lxc')
-        with open(filepath, 'wb') as f:
+        with open(filepath, 'w') as f:
             f.write(content)
 
 
@@ -264,8 +261,8 @@ def untar(tarpath, outfolder, owners=None, overwrite=True, fixperms=True):
     outfolder will be deleted before the new one is put in place. If outfolder
     already exists and overwrite=False, IOError will be raised.
     """
-    # make a folder to untar to 
-    with tmpdir() as here:
+    # make a folder to untar to
+    with tmpdir():
         _, _, ext = tarpath.rpartition('.')
 
         if ext not in ('gz', 'bz2'):
@@ -287,7 +284,7 @@ def untar(tarpath, outfolder, owners=None, overwrite=True, fixperms=True):
                     path = os.path.join(root, d)
                     # chown user:group
                     os.chown(path, user.pw_uid, group.gr_gid)
-                    # chmod ug+xr 
+                    # chmod ug+xr
                     st = os.stat(path)
                     os.chmod(path, st.st_mode | stat.S_IXUSR
                                               | stat.S_IXGRP
@@ -311,7 +308,7 @@ def untar(tarpath, outfolder, owners=None, overwrite=True, fixperms=True):
             else:
                 raise IOError(('Cannot untar %s because %s already exists and '
                               'overwrite=False') % (tarfile, outfolder))
-        os.rename('contents', outfolder)
+        shutil.move('contents', outfolder)
 
 
 def ensure_file(url, path, md5sum=None):
@@ -328,13 +325,13 @@ def ensure_file(url, path, md5sum=None):
 
 
 def download_file(url, path):
-    with tmpdir() as here:
-        print "Downloading %s" % url
+    with tmpdir():
+        print("Downloading %s" % url)
         base = os.path.basename(path)
         with open(base, 'wb') as f:
             resp = requests.get(url, stream=True)
             shutil.copyfileobj(resp.raw, f)
-        os.rename(base, path)
+        shutil.move(base, path)
 
 
 def get_template(name):
@@ -342,6 +339,5 @@ def get_template(name):
     Look for 'name' in the vr.runners.templates folder.  Return its contents.
     """
     path = pkg_resources.resource_filename('vr.runners', 'templates/' + name)
-    with open(path, 'rb') as f:
+    with open(path, 'r') as f:
         return f.read()
-
